@@ -34,7 +34,7 @@ except ImportError:
     fastnumbers = None
 import numpy as np
 from datamatrix import DataMatrix, SeriesColumn, operations
-from eyelinkparser import sample, fixation, defaulttraceprocessor
+from eyelinkparser import sample, fixation, blink, defaulttraceprocessor
 
 ANY_VALUE = int, float, basestring
 ANY_VALUES = list, int, float, basestring
@@ -52,7 +52,11 @@ class EyeLinkParser(object):
         phasefilter=None,
         trialphase=None,
         edf2asc_binary=u'edf2asc',
-        multiprocess=False
+        multiprocess=False,
+        asc_encoding=None,
+        pupil_size=True,
+        gaze_pos=True,
+        time_trace=True
     ):
 
         """
@@ -61,13 +65,13 @@ class EyeLinkParser(object):
 
         keywords:
             folder:
-                type:	str
-                desc:	The folder containing data files
+                type:   str
+                desc:   The folder containing data files
             ext:
-                type:	str, tuple
-                desc:	The data-file extension, or tuple of extensions.
+                type:   str, tuple
+                desc:   The data-file extension, or tuple of extensions.
             downsample:
-                type:	[int, None]
+                type:   [int, None]
                 desc: >
                         Indicates whether traces (if any) should be downsampled.
                         For example, a value of 10 means that the signal becomes
@@ -77,12 +81,12 @@ class EyeLinkParser(object):
                         therefore not be used in combination with the
                         traceprocessor argument.
             maxtracelen:
-                type:	[int, None]
-                desc:	A maximum length for traces. Longer traces are truncated
+                type:   [int, None]
+                desc:   A maximum length for traces. Longer traces are truncated
                         and a UserWarning is emitted. This length refers to the
                         trace after processing.
             traceprocessor:
-                type:	[callable, None]
+                type:   [callable, None]
                 desc: >
                         A function that is applied to each trace before the
                         trace is written to the SeriesColumn. This can be used
@@ -99,7 +103,7 @@ class EyeLinkParser(object):
                         convenience function that applies blink correction and
                         downsampling.
             trialphase:
-                type:	[str, None]
+                type:   [str, None]
                 desc: >
                         Indicates the name of a phase that should be
                         automatically started when the trial starts, or `None`
@@ -108,25 +112,51 @@ class EyeLinkParser(object):
                         of a single long epoch, or when no `start_phase`
                         messages are written to the log file.
             phasefilter:
-                type:	[callable,None]
+                type:   [callable,None]
                 desc: >
                         A function that receives a phase name as argument, and
                         returns a bool indicating whether that phase should be
                         retained.
             edf2asc_binary:
-                type:	str
+                type:   str
                 desc: >
                         The name of the edf2asc executable, which if available
                         can be used to automatically convert edf files to asc.
             multiprocess:
-                type:	[bool, int, None]
-                desc:	>
+                type:   [bool, int, None]
+                desc: >
                         Indicates whether each file should be processed in a
                         different process. This can speed up parsing
                         considerably. If it's not False, it should be an int to
                         indicate the number of processes, or None to indicate
                         that the number of processes should be the same as the
                         number of cores.
+            asc_encoding:
+                type:   [str, None]
+                desc: >
+                        Indicates the character encoding of the `.asc` files,
+                        or `None` to use system default.
+            pupil_size:
+                type:   [bool]
+                desc: >
+                        Indicates whether pupil-size traces should be stored.
+                        If enabled, pupil size is stored as `ptrace_[phase]`
+                        columns.
+            gaze_pos:
+                type:   [bool]
+                desc: >
+                        Indicates whether horizontal and vertical gaze-position
+                        traces should be stored. If enabled, gaze position is
+                        stored as `xtrace_[phase]` and `ytrace_[phase]`
+                        columns.
+            time_trace:
+                type:   [bool]
+                desc: >
+                        Indicates whether timestamp traces should be stored,
+                        which indicate the timestamps of the corresponding
+                        pupil and gaze-position traces. If enabled, timestamps
+                        are stored as `ptrace_[phase]`
+                        columns.
         """
 
         self.dm = DataMatrix()
@@ -140,7 +170,11 @@ class EyeLinkParser(object):
         self._phasefilter = phasefilter
         self._edf2asc_binary = edf2asc_binary
         self._trialphase = trialphase
+        self._asc_encoding = asc_encoding
         self._temp_files = []
+        self._pupil_size = pupil_size
+        self._gaze_pos = gaze_pos
+        self._time_trace = time_trace
         # Get a list of input files. First, only files in the data folder that
         # match any of the extensions. Then, these files are passed to the
         # converter which may return multiple files, for example if they have
@@ -238,7 +272,7 @@ class EyeLinkParser(object):
         self.on_start_file()
         ntrial = 0
         self._linestack = []
-        with open(path) as f:
+        with open(path, encoding=self._asc_encoding) as f:
             for line in self.stacked_file(f):
                 # Only messages can be start-trial messages, so performance we
                 # don't do anything with non-MSG lines.
@@ -334,22 +368,32 @@ class EyeLinkParser(object):
         self.fixylist = []
         self.fixstlist = []
         self.fixetlist = []
+        self.blinkstlist = []
+        self.blinketlist = []
         self._t_onset = self.trialdm['t_onset_%s' % self.current_phase] = l[1]
 
     def end_phase(self, l):
 
         self.trialdm['t_offset_%s' % self.current_phase] = l[1]
         for i, (tracelabel, prefix, trace) in enumerate([
-                (u'pupil', u'ptrace_', self.ptrace),
-                (u'xcoor', u'xtrace_', self.xtrace),
-                (u'ycoor', u'ytrace_', self.ytrace),
-                (u'time', u'ttrace_', self.ttrace),
-                (None, u'fixxlist_', self.fixxlist),
-                (None, u'fixylist_', self.fixylist),
-                (None, u'fixstlist_', self.fixstlist),
-                (None, u'fixetlist_', self.fixetlist),
-                ]):
-            trace = np.array(trace)
+            (u'pupil', u'ptrace_', self.ptrace),
+            (u'xcoor', u'xtrace_', self.xtrace),
+            (u'ycoor', u'ytrace_', self.ytrace),
+            (u'time', u'ttrace_', self.ttrace),
+            (None, u'fixxlist_', self.fixxlist),
+            (None, u'fixylist_', self.fixylist),
+            (None, u'fixstlist_', self.fixstlist),
+            (None, u'fixetlist_', self.fixetlist),
+            (None, u'blinkstlist_', self.blinkstlist),
+            (None, u'blinketlist_', self.blinketlist),
+        ]):
+            if tracelabel == 'pupil' and not self._pupil_size:
+                continue
+            if tracelabel in ('xcoor', 'ycoor') and not self._gaze_pos:
+                continue
+            if tracelabel == 'time' and not self._time_trace:
+                continue
+            trace = np.array(trace, dtype=float)
             if tracelabel is not None and self._traceprocessor is not None:
                 trace = self._traceprocessor(tracelabel, trace)
             if self._maxtracelen is not None \
@@ -362,8 +406,9 @@ class EyeLinkParser(object):
                 len(trace), defaultnan=True)
             self.trialdm[colname][0] = trace
             # Start the time trace at 0
-            if len(trace) and prefix in (u'ttrace_', u'fixstlist_',
-                    u'fixetlist_'):
+            if len(trace) and prefix in ('ttrace_', 'fixstlist_',
+                                         'fixetlist_', 'blinkstlist',
+                                         'blinketlist'):
                 self.trialdm[colname][0] -= self._t_onset
         # DEBUG CODE
         # 	from matplotlib import pyplot as plt
@@ -388,6 +433,11 @@ class EyeLinkParser(object):
         self.fixylist.append(f.y)
         self.fixstlist.append(f.st)
         self.fixetlist.append(f.et)
+        
+    def parse_blink(self, b):
+        
+        self.blinkstlist.append(b.st)
+        self.blinketlist.append(b.et)
 
     def parse_phase(self, l):
 
@@ -413,6 +463,9 @@ class EyeLinkParser(object):
         f = fixation(l)
         if f is not None:
             self.parse_fixation(f)
+        b = blink(l)
+        if b is not None:
+            self.parse_blink(b)
 
     def is_start_trial(self, l):
 
